(* Beginnings of Flatten *)

(* Authors: Hannah Pierce-Hoffman, Kundan Guha, Yuanxia Lee*)

(* Open Ast and Netlist modules so we can use their data structures *)
module A = Ast
module N = Netlist 

module StringMap = Map.Make(String) 

(* Initialize empty list to build the netlist *)
let nl = []

(* Helper function to convert AST prims to Netlist prims.
Need to separately deal with Not/Neg conversion case later *)
let prim_converter aop = 
	match aop with
		A.And -> N.And
	  | A.Or -> N.Or

let expr_converter e =
    match e with
        A.Name s -> N.Name s

(* Arguments:
vset_assign_list is a list of A.Vset and A.Assign types
vdecmap is a StringMap of names to A.Vdec types
gatemap is a StringMap of names to netlist-style gate definitions
(you only need to look something up in gatemap if it is a user-defined
gate and not a primitive)
*)
let flat vset_assign_list vdecmap gatemap = 
    
    (* The vs_lookup function takes a single A.Vset item and
    looks up its name in vdecmap.  If the name is found to 
	represent a prim, a list containing a single netlist-ready N.Expr.Prim 
	object is returned.  If the name is found to represent a user-defined
	gate, we look up the gate in gatemap, then return
	the relevant list of netlist definitions that represent this
	user-defined gate.  In either case, vs_lookup returns a list of
	netlist-ready statements that can then be concatenated with the
	growing netlist. *)
    let vs_lookup vs = 
    	match vs with
    		A.Vset(vname, inp_list) ->
    			let found_vdecl = StringMap.find vname vdecmap in 
    			match found_vdecl with
    				A.Pdec (prtype, p_vname) ->
    					match inp_list with
    					[in1; in2] ->
    						let nlop = prim_converter prtype in
    						(* Possible concern: types of in1 and in2 *)
    						let nl_ready_stmt = N.Expr(vname, N.Prim(nlop, expr_converter in1, expr_converter in2))
    					in [nl_ready_stmt]
                        | [in1] ->
                            let nl_ready_stmt = N.Expr(vname, N.Neg(expr_converter in1))
                        in [nl_ready_stmt]

    			  (* For Gdec, the gatebody is assumed to be a list of netlist-ready statements,
    			  so we just return this and don't worry further about the inputs.
    			  | A.Gdec (custom_gtype, g_vname) ->
    					let found_gatebody = StringMap.find vname gatemap in
    					found_gatebody *)

    (* Call vs_lookup on every item in vset_assign_list using List.map and save the end result
    in nl_ready_list*)
    in
    let list_of_lists = List.map vs_lookup vset_assign_list in
    List.concat list_of_lists

    (* Desired result: nl_ready_list is a list of N.Expr, ready for codegen *)







