/* Authors: Kundan Guha, Yuanxia Lee, Sarah Walker, Hannah Pierce-Hoffman */

%{
open Ast
%}


%token ONE ZERO COMMA OUT
%token RCBRACE LCBRACE RSBRACE LSBRACE RPAREN LPAREN
%token ASSIGN AND OR SEMI NOT BOOL
%token EOF
%token <string> ID GATE
%token <int> LITERAL

%start program
%type <Ast.gate_def> program

%%

program:
    EOF { {name = ""; inp = []; body = []; out=Arr [];} }
  | gate_def EOF { $1 }

gate_def:
    GATE LPAREN id_list RPAREN LCBRACE stmt_list OUT ASSIGN vexpr SEMI RCBRACE
      { {name = $1; inp = $3; body = $6; out = $9;} }

id_list:
    /* Nothing */ { [] }
  | nonempt_id_list { List.rev $1 }

nonempt_id_list:
    ID { [$1] }
  | nonempt_id_list COMMA ID { $3 :: $1 }

stmt_list:
    /* Nothing */ { [] }
  | stmtl { List.rev $1 } /* Entirely to allow left recursion */

stmtl:
    stmt SEMI { [$1] }
  | gate_def { [Gdef $1] }
  | typ ID LPAREN exprl RPAREN SEMI { [Vset ($2, $4); Vdec ($1 $2)] }
  | stmtl stmt SEMI { $2 :: $1 } /* Left recursive */
  | stmtl typ ID LPAREN exprl RPAREN SEMI { List.append [Vset ($3, $5); Vdec ($2 $3)] $1  }
  | stmtl gate_def /*No SEMI*/ { (Gdef $2) :: $1 }
  /* Wasn't sure of a more elegant way to deal with that case */

stmt:
    typ ID { Vdec ($1 $2) }
  | ID LPAREN exprl RPAREN { Vset ($1, $3) }
  | ID ASSIGN vexpr { Assign (Symb $1, $3) }
  | ID LSBRACE num RSBRACE ASSIGN vexpr { Assign ((Deref ($1, $3)), $6) }

typ:
    GATE { fun x -> Gdec ($1, x) }
  | prim { fun x -> Pdec ($1, x) }
  | BOOL { fun x -> Bdec x }
  | BOOL LSBRACE num RSBRACE { fun x -> Barr (x, $3) }

prim:
    AND { And }
  | OR { Or }
  | NOT { Not }
/*  | DFF { Dff } */

num:
    ONE { 1 }
  | ZERO { 0 }
  | LITERAL { $1 }

vexpr:
    expr { $1 }
  | LSBRACE exprl RSBRACE { Arr $2 }

exprl:
    /* Nothing */ { [] }
  | expr { [$1] }
  | expr COMMA exprl { $1 :: $3 }

expr:
    ID { Name $1 }
  | ONE { Bit true }
  | ZERO { Bit false}
  | ID LSBRACE num RSBRACE { Index ($1, $3) }

