(* Authors: Kundan Guha, Yuanxia Lee *)
{
open Parser
(* Get error messages with line numbers (format found in an online ocamllex tutorial)*)
let line_num = ref 1
exception Syntax_error of string
let syntax_error msg = raise (Syntax_error (msg ^ " on line " ^ (string_of_int !line_num)))
}

let digit = ['0'-'9']
let digits = digit+
let cap = ['A'-'Z']
let low = ['a'-'z']
let alpha = (cap | low)


rule token = parse
    [' ' '\t' '\r' ] { token lexbuf }
  | '\n' { incr line_num; token lexbuf }
  | '%' { singlcomment lexbuf }
  | "%%" { multicomment lexbuf }
  | "out" { OUT }
  | '0' { ZERO }
  | '1' { ONE }
  | "AND" { AND }
  | "OR" { OR }
  | "NOT" { NOT }
(*  | "DFF" { DFF } *)
  | "bool" { BOOL }
  | cap (cap | digit | '_')* as lit { GATE(lit) }
  | low (alpha | digit | '_')* as lit { ID(lit) }
(*  | cap (alpha | digit)* as lit { DECORATOR(lit) } *)
  | digits as lit { LITERAL(int_of_string lit) }
  | '{' { LCBRACE }
  | '}' { RCBRACE }
  | '[' { LSBRACE }
  | ']' { RSBRACE }
  | '(' { LPAREN }
  | ')' { RPAREN }
  | ';' { SEMI }
  | ',' { COMMA }
  | '=' { ASSIGN }
  | _ as lit { syntax_error ("Unrecognised token: " ^ (String.make 1 lit)) }
  | eof { EOF }
and singlcomment = parse
  '\n' { token lexbuf }
 | _   { singlcomment lexbuf }
and multicomment = parse
  "%%" { token lexbuf }
| _    { multicomment lexbuf }
